# -*- coding: utf-8 -*-
"""A python module to interact with elabftw API"""

__version__ = "0.5.0"
__author__ = "Nicolas CARPi"
__author_email__ = "nicolas.carpi@curie.fr"
__license__ = "GPL v3"
__copyright__ = "Copyright (©) 2017 Nicolas CARPi"

from .Manager import Manager
from .baseapi import Error, SetupError, DataReadError
